﻿using Microsoft.Graphics.Canvas.Effects;
using Microsoft.Practices.ServiceLocation;
using TVShowTime.UWP.ViewModels;
using Windows.ApplicationModel.Core;
using Windows.Foundation.Metadata;
using Windows.UI;
using Windows.UI.Composition;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Hosting;
using Windows.UI.Xaml.Navigation;

namespace TVShowTime.UWP.Views
{
    public sealed partial class LoginPage : Page
    {
        #region Properties

        public LoginViewModel ViewModel { get; }

        #endregion

        #region Constructor

        public LoginPage()
        {
            InitializeComponent();

            ViewModel = (LoginViewModel)DataContext;

            InitializeFrostedGlass(FrostedGlassPanel);

            Loaded += (sender, e) =>
            {
                ServiceLocator.Current.GetInstance<LoginViewModel>().TryAuthenticate();
            };
        }

        #endregion

        #region Events

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
            
            // Style title bar (Desktop)
            if (ApiInformation.IsTypePresent("Windows.ApplicationModel.Core"))
            {
                CoreApplication.GetCurrentView().TitleBar.ExtendViewIntoTitleBar = false;
            }

            // Each time a navigation event occurs, update the Back button's visibility
            SystemNavigationManager.GetForCurrentView().AppViewBackButtonVisibility = AppViewBackButtonVisibility.Collapsed;
        }

        #endregion

        #region Methods

        private void InitializeFrostedGlass(UIElement glassHost)
        {
            // Check if CompositionBackdropBrush is supported
            if (!ApiInformation.IsTypePresent("Windows.UI.Composition.CompositionBackdropBrush"))
            {
                return;
            }

            var hostVisual = ElementCompositionPreview.GetElementVisual(glassHost);
            var compositor = hostVisual.Compositor;

            // Create a glass effect, requires Win2D NuGet package
            var glassEffect = new GaussianBlurEffect
            {
                BlurAmount = 15.0f,
                BorderMode = EffectBorderMode.Hard,
                Source = new ArithmeticCompositeEffect
                {
                    MultiplyAmount = 0,
                    Source1Amount = 0.75f,
                    Source2Amount = 0.25f,
                    Source1 = new CompositionEffectSourceParameter("backdropBrush"),
                    Source2 = new ColorSourceEffect
                    {
                        Color = Color.FromArgb(255, 245, 245, 245)
                    }
                }
            };

            //  Create an instance of the effect and set its source to a CompositionBackdropBrush
            var effectFactory = compositor.CreateEffectFactory(glassEffect);
            var backdropBrush = compositor.CreateBackdropBrush();
            var effectBrush = effectFactory.CreateBrush();

            effectBrush.SetSourceParameter("backdropBrush", backdropBrush);

            // Create a Visual to contain the frosted glass effect
            var glassVisual = compositor.CreateSpriteVisual();
            glassVisual.Brush = effectBrush;

            // Add the blur as a child of the host in the visual tree
            ElementCompositionPreview.SetElementChildVisual(glassHost, glassVisual);

            // Make sure size of glass host and glass visual always stay in sync
            var bindSizeAnimation = compositor.CreateExpressionAnimation("hostVisual.Size");
            bindSizeAnimation.SetReferenceParameter("hostVisual", hostVisual);

            glassVisual.StartAnimation("Size", bindSizeAnimation);
        }

        #endregion
    }
}
